import django_filters
from django.test import override_settings
from django.urls import reverse
from model_bakery import baker
from rest_framework import status
from rest_framework.test import APITestCase

from django_audit_events.models import AuditEvent
from django_audit_events.tests.utils.models import Poll
from django_audit_events.views import AuditEventViewSet

if django_filters.VERSION < (2, 0):
    from rest_framework_filters.backends import DjangoFilterBackend
else:
    from django_filters.rest_framework.backends import DjangoFilterBackend


class AuditEventViewSetTestCase(APITestCase):
    def setUp(self):
        self.poll_1 = baker.make(Poll)
        self.poll_2 = baker.make(Poll)
        self.audit_event_1 = baker.make(AuditEvent, content_object=self.poll_1)
        self.audit_event_2 = baker.make(AuditEvent, content_object=self.poll_2)

    @override_settings(ROOT_URLCONF="django_audit_events.urls")
    def test_get_events(self):
        response = self.client.get(reverse("auditevent-list"))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 2)

    @override_settings(ROOT_URLCONF="django_audit_events.urls")
    def test_get_events_with_object_id(self):
        # Added for filtering
        AuditEventViewSet.filter_backends = [DjangoFilterBackend]

        response = self.client.get(
            reverse("auditevent-list") + "?object_id={}".format(self.poll_1.pk)
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 1)



from django.contrib.auth import get_user_model
from rest_framework import serializers

from django_audit_events.models import get_audit_event_model


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = get_user_model()
        fields = ("pk", "email", "first_name", "last_name")


class EventSerializer(serializers.ModelSerializer):
    user = UserSerializer()

    class Meta:
        fields = "__all__"
        model = get_audit_event_model()



import django_filters

from django_audit_events.models import get_audit_event_model

if django_filters.VERSION < (2, 0):
    import rest_framework_filters as PY2Filters

    class AuditEventRestFrameworkFilter(PY2Filters.FilterSet):
        user = PY2Filters.NumberFilter(name="user")
        content_type = PY2Filters.NumberFilter(name="content_type")
        object_id = PY2Filters.NumberFilter(name="object_id")
        timestamp__lt = PY2Filters.DateTimeFilter(name="timestamp", lookup_expr="lt")
        timestamp__gt = PY2Filters.DateTimeFilter(name="timestamp", lookup_expr="gt")
        timestamp__lte = PY2Filters.DateTimeFilter(name="timestamp", lookup_expr="lte")
        timestamp__gte = PY2Filters.DateTimeFilter(name="timestamp", lookup_expr="gte")

        class Meta:
            model = get_audit_event_model()
            fields = ("user", "content_type", "object_id", "timestamp")

    AuditEventFilterSet = AuditEventRestFrameworkFilter

else:
    from django_filters import FilterSet, filters

    class AuditEventDjangoFilter(FilterSet):
        user = filters.NumberFilter(field_name="user")
        content_type = filters.NumberFilter(field_name="content_type")
        object_id = filters.NumberFilter(field_name="object_id")
        timestamp__lt = filters.DateTimeFilter(field_name="timestamp", lookup_expr="lt")
        timestamp__gt = filters.DateTimeFilter(field_name="timestamp", lookup_expr="gt")
        timestamp__lte = filters.DateTimeFilter(
            field_name="timestamp", lookup_expr="lte"
        )
        timestamp__gte = filters.DateTimeFilter(
            field_name="timestamp", lookup_expr="gte"
        )

        class Meta:
            model = get_audit_event_model()
            fields = ("user", "content_type", "object_id", "timestamp")

    AuditEventFilterSet = AuditEventDjangoFilter



from __future__ import unicode_literals

import django

if django.VERSION >= (2, 0, 0):
    from django.utils.translation import gettext_lazy as _
else:
    from django.utils.translation import ugettext_lazy as _

from django.apps import AppConfig


class DjangoAuditEventsConfig(AppConfig):
    name = "django_audit_events"
    verbose_name = _("Django Audit Event")



try:
    from django.urls import include, path
except ImportError:
    from django.conf.urls import include, url as path
from rest_framework.routers import DefaultRouter

from django_audit_events.views import AuditEventViewSet

router = DefaultRouter()
router.register(r"events", AuditEventViewSet)

app_name = "django_audit_events"

urlpatterns = [path("", include(router.urls))]



# -*- coding: utf-8 -*-
# Generated by Django 3.0.11 on 2022-09-20 13:27

from django.db import migrations


class Migration(migrations.Migration):
    atomic = False

    dependencies = [
        ('django_audit_events', '0005_auto_20220215_0756'),
    ]

    operations = [
        migrations.RunSQL(
            sql='CREATE INDEX CONCURRENTLY IF NOT EXISTS '
                '"django_audit_events_auditevent_timestamp_0c865855_uniq" '
                'ON "django_audit_events_auditevent" ("timestamp");',
            reverse_sql='DROP INDEX CONCURRENTLY IF EXISTS '
                        '"django_audit_events_auditevent_timestamp_0c865855_uniq";'
        ),
    ]



from django.db.models import ForeignObject, ManyToManyField
from rest_framework import viewsets

from django_audit_events.filters import AuditEventFilterSet
from django_audit_events.models import get_audit_event_model
from django_audit_events.serializers import EventSerializer


class AuditEventViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = get_audit_event_model().objects.all()
    filter_class = AuditEventFilterSet  # for older DRF - django-filter version
    filterset_class = AuditEventFilterSet
    serializer_class = EventSerializer

    def get_queryset(self):
        queryset = super(AuditEventViewSet, self).get_queryset()
        # noinspection PyProtectedMember
        fields = queryset.model._meta.local_fields
        select_related = []
        prefetch_related = []

        for field in fields:
            if isinstance(field, ForeignObject):
                select_related.append(field.name)
            elif isinstance(field, ManyToManyField):
                prefetch_related.append(field.name)

        return queryset.select_related(*select_related).prefetch_related(
            *prefetch_related
        )
